!==========================================================================!
!                             DIMENSIONS                                   !
!==========================================================================!

BEGIN_PROVIDER [ double precision, E_kin_elec_SVD, (elec_num) ]
  implicit none
  BEGIN_DOC
  ! Electronic Kinetic energy : -1/2 (Lapl.Psi_SVD)/Psi_SVD
  END_DOC
  integer :: i
  do i = 1, elec_num
    E_kin_elec_SVD(i) = -0.5d0 * psi_lapl_psi_inv_SVD(i)
  enddo
END_PROVIDER



!==========================================================================!
!                             PROPERTIES                                   !
!==========================================================================!


BEGIN_PROVIDER  [ double precision, E_loc_SVD ]
  implicit none
  include '../types.F'
  BEGIN_DOC
  ! Local energy : E_kin + E_pot + E_nucl
  END_DOC

  integer :: i

  E_loc_SVD = E_nucl
  !DIR$ VECTOR ALIGNED
  !DIR$ LOOP COUNT(200)
  do i = 1, elec_num
    E_loc_SVD += E_kin_elec_SVD(i) + E_pot_elec(i)
  enddo

  ! Avoid divergence of E_loc_SVD and population explosion
  if (do_pseudo) then
    double precision :: delta_e
    E_loc_SVD = max(2.d0*E_ref, E_loc_SVD)
  endif

  E_loc_SVD_min = min(E_loc_SVD,E_loc_SVD_min)
  E_loc_SVD_max = max(E_loc_SVD,E_loc_SVD_max)
  SOFT_TOUCH E_loc_SVD_min E_loc_SVD_max

END_PROVIDER
