 BEGIN_PROVIDER [ double precision, psi_value_SVD ]
  implicit none
  BEGIN_DOC
  ! Value of the wave function
  END_DOC
  psi_value_SVD = psidet_value_SVD * jast_value
  if (psi_value_SVD == 0.d0) then
    call abrt(irp_here,"Value of the wave function is 0.")
  endif
END_PROVIDER



BEGIN_PROVIDER [ double precision, psi_value_inv_SVD ]
  implicit none
  BEGIN_DOC
  ! 1. / psi_value_SVD
  END_DOC
  psi_value_inv_SVD = 1.d0 / psi_value_SVD
END_PROVIDER



BEGIN_PROVIDER [ double precision, psi_value_inv2_SVD ]
 implicit none
 BEGIN_DOC
 ! 1./(psi_value_SVD)**2
 END_DOC
 psi_value_inv2_SVD = psi_value_inv_SVD * psi_value_inv_SVD
END_PROVIDER



BEGIN_PROVIDER [ double precision, psi_lapl_SVD, (elec_num_8) ]
  implicit none
  BEGIN_DOC
  ! Laplacian of the wave function
  END_DOC
  integer :: i, j
  !DIR$ VECTOR ALIGNED
  !DIR$ LOOP COUNT (100)
  do j = 1, elec_num
    psi_lapl_SVD(j) = jast_value * (                           &
        psidet_grad_lapl_SVD(4,j) +                            &
        psidet_value_SVD * jast_lapl_jast_inv(j) + 2.d0 * (    &
        psidet_grad_lapl_SVD(1,j) * jast_grad_jast_inv_x(j) +  &
        psidet_grad_lapl_SVD(2,j) * jast_grad_jast_inv_y(j) +  &
        psidet_grad_lapl_SVD(3,j) * jast_grad_jast_inv_z(j) ) )
  enddo
END_PROVIDER



 BEGIN_PROVIDER [ double precision, psi_grad_psi_inv_x_SVD, (elec_num_8) ]
&BEGIN_PROVIDER [ double precision, psi_grad_psi_inv_y_SVD, (elec_num_8) ]
&BEGIN_PROVIDER [ double precision, psi_grad_psi_inv_z_SVD, (elec_num_8) ]
  implicit none
  BEGIN_DOC
  ! grad(psi_SVD) / psi_SVD
  END_DOC
  integer :: j
  !DIR$ VECTOR ALIGNED
  !DIR$ LOOP COUNT (100)
  do j = 1, elec_num
    psi_grad_psi_inv_x_SVD(j) = psidet_grad_lapl_SVD(1,j) * psidet_inv_SVD + jast_grad_jast_inv_x(j)
    psi_grad_psi_inv_y_SVD(j) = psidet_grad_lapl_SVD(2,j) * psidet_inv_SVD + jast_grad_jast_inv_y(j)
    psi_grad_psi_inv_z_SVD(j) = psidet_grad_lapl_SVD(3,j) * psidet_inv_SVD + jast_grad_jast_inv_z(j)
  enddo 
END_PROVIDER



BEGIN_PROVIDER [ double precision, psi_lapl_psi_inv_SVD, (elec_num_8) ]
  implicit none
  BEGIN_DOC
  ! (Laplacian psi_SVD) / psi_SVD
  END_DOC
  integer :: i, j
  !DIR$ VECTOR ALIGNED
  !DIR$ LOOP COUNT (100)
  do j = 1, elec_num
    psi_lapl_psi_inv_SVD(j) = psi_lapl_SVD(j) * psi_value_inv_SVD
  enddo
END_PROVIDER

